==> parts-0.1.3-20180901.235711-1.jar: parts/components/ring.clj <==
(ns parts.components.ring
  (:require
   [clojure.spec.alpha :as s]
   [parts.components.middleware :as cptmdw]
   [com.stuartsierra.component :as c]))

;; -----------------------------------------------------
;; ring binding spec
;; -----------------------------------------------------

(s/def ::binding-fn
  fn?)

;; -----------------------------------------------------
;; protocols
;; -----------------------------------------------------

(defprotocol IRequestHandler
  (-request-handler [this]))

(defn request-handler
  [irequest-handler]
  (-request-handler irequest-handler))

(defprotocol IRequestBinding
  (-request-binding [this]))

(defn request-binding
  [irequest-binding]
  (-request-binding irequest-binding))

;; -----------------------------------------------------
;; ring head
;; -----------------------------------------------------

(defrecord RingHead [ring-handler ring-middleware]
  IRequestHandler
  (-request-handler [this]
    (let [handler    (request-handler ring-handler)
          middleware (if (some? ring-middleware)
                       (cptmdw/wrapper ring-middleware)
                       identity)]
      (middleware
       (fn [request]
         (let [bindings (into {}
                              (comp
                               (map val)
                               (filter #(satisfies? IRequestBinding %))
                               (map request-binding))
                              this)]
           (handler (assoc request :bindings bindings))))))))

(defn make-ring-head
  []
  (c/using
   (map->RingHead {})
   [:ring-handler]))

;; -----------------------------------------------------
;; ring binding
;; -----------------------------------------------------

(defrecord RingBinding [binding-fn]
  IRequestBinding
  (-request-binding [this]
    (binding-fn this)))

(defn make-ring-binding
  [binding-fn]
  (->RingBinding (s/assert ::binding-fn binding-fn)))

==> parts-0.1.5-20180904.041259-1.jar: parts/components/ring.clj <==
(ns parts.components.ring
  (:require
   [clojure.spec.alpha :as s]
   [parts.components.middleware :as cptmdw]
   [com.stuartsierra.component :as c]))

;; -----------------------------------------------------
;; ring binding spec
;; -----------------------------------------------------

(s/def ::binding-fn
  fn?)

;; -----------------------------------------------------
;; protocols
;; -----------------------------------------------------

(defprotocol IRequestHandler
  (-request-handler [this]))

(defn request-handler
  [irequest-handler]
  (-request-handler irequest-handler))

(defprotocol IRequestBinding
  (-request-binding [this]))

(defn request-binding
  [irequest-binding]
  (-request-binding irequest-binding))

;; -----------------------------------------------------
;; ring head
;; -----------------------------------------------------

(defrecord RingHead [ring-handler ring-middleware]
  IRequestHandler
  (-request-handler [this]
    (let [handler    (request-handler ring-handler)
          middleware (if (some? ring-middleware)
                       (cptmdw/wrapper ring-middleware)
                       identity)
          wrapped    (middleware handler)]
      (fn [request]
        (let [bindings (into {}
                             (map (fn [[k v]]
                                    [k (if (satisfies? IRequestBinding v)
                                         (request-binding v)
                                         v)]))
                             this)]
          (wrapped (assoc request :bindings bindings)))))))

(defn make-ring-head
  []
  (c/using
   (map->RingHead {})
   [:ring-handler]))
